; ========================================================================== ;
;                            .: Stack 'Em! :.                                ;
;             4K Intellivision game for the MiniGame Compo 2004              ;
; -------------------------------------------------------------------------- ;
;                      Mail : a_chevallier@yahoo.com                         ;
;                       Web : http://intyos.free.fr                          ;
; ========================================================================== ;
;  This program is free software; you can redistribute it and/or modify      ;
;  it under the terms of the GNU General Public License as published by      ;
;  the Free Software Foundation; either version 2 of the License, or         ;
;  (at your option) any later version.                                       ;
;                                                                            ;
;  This program is distributed in the hope that it will be useful,           ;
;  but WITHOUT ANY WARRANTY; without even the implied warranty of            ;
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU          ;
;  General Public License for more details.                                  ;
;                                                                            ;
;  You should have received a copy of the GNU General Public License         ;
;  along with this program; if not, write to the Free Software               ;
;  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 ;
; ========================================================================== ;
;                   Copyright (c) 2004, Arnauld Chevallier                   ;
; ========================================================================== ;

; -------------------------------------------------------------------------- ;
; 16-bit ROM                                                                 ;
; -------------------------------------------------------------------------- ;

            ROMW        16

; -------------------------------------------------------------------------- ;
; Macros                                                                     ;
; -------------------------------------------------------------------------- ;

            INCLUDE     "stack_em.mac"

; -------------------------------------------------------------------------- ;
; System addresses                                                           ;
; -------------------------------------------------------------------------- ;

MOBX        EQU         $0000                   ; MOB registers
MOBY        EQU         $0008
MOBA        EQU         $0010
MOBC        EQU         $0018

HDELAY      EQU         $0030                   ; horizontal delay
VDELAY      EQU         $0031                   ; vertical delay
B_EXT       EQU         $0032                   ; border extension

VBLK0       EQU         $0100                   ; VBlank
VBLK1       EQU         $0101

BT          EQU         $0200                   ; BackTab

RET_INT     EQU         $1014                   ; EXEC : return from interrupt
EXEC_RND    EQU         $169E                   ; EXEC : get random number
EXEC_CLEAR  EQU         $1738                   ; EXEC : clear memory
EXEC_KEY    EQU         $14C3                   ; EXEC : wait for key pressed/released

GRAM        EQU         $3800                   ; GRAM

; -------------------------------------------------------------------------- ;
; Variables                                                                  ;
; -------------------------------------------------------------------------- ;

; $0102 - $01EF
; --------------------------------------------------------------------------

FRAME       EQU         $0102

C0          EQU         $0103                   ; colors of falling brick
C1          EQU         $0104
C2          EQU         $0105

CN0         EQU         $0106                   ; colors of next brick
CN1         EQU         $0107
CN2         EQU         $0108

X           EQU         $0109                   ; position of falling brick
Y           EQU         $010A

HC_CNT      EQU         $010B                   ; counter for HC inputs
FAST_OK     EQU         $010C                   ; fast fall enabled

RND1        EQU         $010D                   ; random values between 0...4
RND2        EQU         $010E
RND3        EQU         $010F

LEVEL       EQU         $0110                   ; level

G_FADE      EQU         $0111                   ; global volume fade

; $0308 - $035F
; --------------------------------------------------------------------------

Y_CNT       EQU         $0318
Y_INC       EQU         $0319

SC_10K      EQU         $031A                   ; score
SCORE       EQU         $031B

GEMS        EQU         $031C                   ; gems

BONUS       EQU         $031D                   ; bonus for chain reactions

LEV_TH      EQU         $031E                   ; threshold for next level

; -------------------------------------------------------------------------- ;
; Main entry point                                                           ;
; -------------------------------------------------------------------------- ;

            ORG         $4800

init        PROC

            DIS                                 ; no interrupts

            MVII        #$2F0,      R6          ; initialize stack
            MVO         R6,         $35E        ; initialize EXEC random seed

            MVII        #vblank,    R0          ; set vblank routine
            MVO         R0,         VBLK0
            SWAP        R0
            MVO         R0,         VBLK1

            EIS                                 ; enable interrupts

            ENDP

; -------------------------------------------------------------------------- ;
; Intro                                                                      ;
; -------------------------------------------------------------------------- ;

intro       PROC

            CALL        init_psg                ; initialize PSG

            CALL        clear_mobs              ; clear MOBs
            CALL        cls                     ; clear screen

            InitGram    (5, 0, gfx)             ; initialize GRAM
            InitGram    (27, 10, gfx.digits)

            CALL        wait_nvbl
            DECLE       1
            MVO         R0,         $0028       ; colorstack : black
            MVO         R0,         $002C       ; black border
            MVI         $0021,      R0          ; colorstack mode

            CALL        text
            DECLE       BT+1,7
            STRING16    "Arnauld Chevallier"
            CALL        text
            DECLE       BT+26,7
            STRING16    "presents"

            MVII        #$08F7,     R0          ; draw title
            CALL        draw
            DECLE       BT+86,4
            CALL        draw
            DECLE       BT+91,3

            CALL        text
            DECLE       BT+142,7
            STRING16    "4K entry for the  "
            BACK
            STRING16    " MiniGame Compo 2K4"

            CALL        text
            DECLE       BT+200,7
            STRING16    "(1)-(9) : level 1..9"
            BACK
            STRING16    "other key : level 10"

            CALL        EXEC_KEY                ; wait for key

            MOVR        R5,         R1
            ANDI        #$FF,       R5
            BNEQ        @@hc_ok

            SWAP        R1

@@hc_ok:    MVII        #keypad,    R4
            CLRR        R2

@@test:     INCR        R2
            CMPI        #10,        R2
            BEQ         @@done
            CMP@        R4,         R1
            BNEQ        @@test

@@done:     MVO         R2,         LEVEL

            CALL        cls
            CALL        wait_nvbl
            DECLE       1
            MVO         R0,         $0021

            ENDP

; -------------------------------------------------------------------------- ;
; New game                                                                   ;
; -------------------------------------------------------------------------- ;

new         PROC

            CALL        clear                   ; clear score
            DECLE       SC_10K, 3

            MVII        #35,        R0          ; initial threshold for next level
            MVO         R0,         LEV_TH

            CALL        upd_next
            CALL        wait_nvbl
            DECLE       1

            ENDP

; -------------------------------------------------------------------------- ;
; Draw board                                                                 ;
; -------------------------------------------------------------------------- ;

board       PROC

            MVII        #$3600,     R0          ; purple background
            MVII        #$3E10,     R1          ; left border
            MVII        #$3E18,     R2          ; right border
            MVII        #BT,        R4          ; R4 = backtab pointer

@@drw_row:  CALL        copy6
            MVO@        R1,         R4
            ADDI        #6,         R4
            MVO@        R2,         R4
            CALL        copy6

            CMPI        #BT+240,    R4
            BNEQ        @@drw_row

;           01234567890123456789
;      000        |      |
;      020  SSS   |      | TTTT
;      040  000000|      |TTT
;      060        |      |
;      080  JJJJ  |      | xxx
;      100  00000 |      | xNx
;      120        |      | xNx
;      140  LLL   |      | xNx
;      160  00    |      | xxx
;      180        |      |
;      200        |      |
;      220        |      |

            MVII        #$3EA7,     R0          ; GRAM card #20

            CALL        draw                    ; 'Score'
            DECLE       BT+20,3

            CALL        draw                    ; 'Jewels'
            DECLE       BT+80,4

            CALL        draw                    ; 'Level'
            DECLE       BT+140,3

            CALL        draw                    ; title
            DECLE       BT+35,4
            CALL        draw
            DECLE       BT+54,3

            CALL        disp_score
            CALL        disp_gems
            CALL        disp_level

            CALL        init_song               ; initialize music
            DECLE       song00

            ENDP

; -------------------------------------------------------------------------- ;
; New brick                                                                  ;
; -------------------------------------------------------------------------- ;

new_brick   PROC

            CLRR        R0                      ; init. Y
            MVO         R0,         Y

            CMP         BT+9,       R0          ; game over ? ...
            BNEQ        game_over

            CALL        upd_next                ; ... no : update next brick

            MVII        #16,        R0          ; init. X
            MVO         R0,         X

            ENDP

; -------------------------------------------------------------------------- ;
; Main loop                                                                  ;
; -------------------------------------------------------------------------- ;

loop        PROC

            MVI         Y,          R2          ; R2 = Y
            MVI         Y_CNT,      R0
            ADD         Y_INC,      R0

            CMPI        #$100,      R0
            BLT         @@sav_cnt

            ADDI        #4,         R2
            SUBI        #$100,      R0

@@sav_cnt:  MVO         R0,         Y_CNT       ; save counter
            MVO         R2,         Y           ; save Y

            SLR         R2,         2           ; R2 = BT pointer
            SLR         R2
            ADDI        #@@y2bt,    R2
            MVI@        R2,         R2

            MVI         X,          R0          ; add X
            SLR         R0,         2
            SLR         R0
            ADDR        R0,         R2

            CMPI        #BT+240,    R2          ; bottom of field ?
            BGE         @@hit

            MVI@        R2,         R0          ; hit something ? ...
            TSTR        R0
            BEQ         @@hc

@@hit:      MVII        #C2,        R3          ; ... yes : copy brick on backtab

@@copy_bt:  SUBI        #20,        R2
            MVI@        R3,         R1
            ADDI        #col.c,     R1
            MVI@        R1,         R1
            MVO@        R1,         R2

            DECR        R3
            CMPI        #C0,        R3
            BGE         @@copy_bt

            CALL        clear_mobs

            CLRR        R0                      ; initial bonus = 0
            MVO         R0,         BONUS

            MVO         R0,         FAST_OK     ; no fast fall

@@align:    MVII        #BT+7,      R1          ; now, look for bricks alignments
            CLRR        R3                      ; R3 = total number of bricks destroyed

@@chk_a:    CALL        align                   ; x++
            DECLE       1
            CALL        align                   ; x-- / y++
            DECLE       19
            CALL        align                   ; y++
            DECLE       20
            CALL        align                   ; x++ / y++
            DECLE       21

@@nxt_a:    INCR        R1

            MVI@        R1,         R5          ; right border ?
            CMPI        #$3E18,     R5
            BNEQ        @@chk_a

            CALL        wait_nvbl
            DECLE       1

            ADDI        #20-6,      R1          ; next row
            CMPI        #BT+247,    R1
            BNEQ        @@chk_a

            TSTR        R3                      ; something destroyed ? ...
            BEQ         new_brick

            MVI         GEMS,       R0          ; ... yes : update gems
            ADDR        R3,         R0
            MVO         R0,         GEMS

            CMP         LEV_TH,     R0
            BLT         @@gems_ok

            MVI         LEV_TH,     R0
            ADDI        #35,        R0
            MVO         R0,         LEV_TH

            MVI         LEVEL,      R0
            INCR        R0
            MVO         R0,         LEVEL

@@gems_ok:  MVII        #20,        R0          ; update score
            MOVR        R0,         R1
            ADD         BONUS,      R0
            MVII        #10,        R2
            SUBI        #2,         R3

@@upd_sc:   ADDR        R2,         R0
            ADDR        R1,         R2
            DECR        R3
            BNEQ        @@upd_sc

            ADD         SCORE,      R0
            MVO         R0,         SCORE

            MVI         BONUS,      R0          ; bonus += 50
            ADDI        #50,        R0
            MVO         R0,         BONUS

            CALL        disp_gems
            CALL        disp_score
            CALL        disp_level

            MVII        #8,         R2          ; process animation

@@upd_c8:   MVII        #gfx.b0,    R4
            MOVR        R2,         R0
            ANDI        #1,         R0
            BEQ         @@upd_g

            MVII        #gfx.b1,    R4

@@upd_g:    PSHR        R2
            CALL        wait_nvbl
            DECLE       5
            PULR        R2

            CALL        load_gram
            DECLE       GRAM+8*8
            DECLE       8

            DECR        R2
            BNEQ        @@upd_c8

            MVII        #$082F,     R0          ; overwrite destroyed bricks
            MVII        #$8000,     R1
            MVII        #BT+7,      R4

@@row_d:    MVII        #6,         R3

@@chk_d:    CMP@        R4,         R1
            BNC         @@destroy

@@nxt_d:    DECR        R3
            BNEQ        @@chk_d

            ADDI        #20-6,      R4
            CMPI        #BT+247,    R4
            BNEQ        @@row_d
            B           @@xpld_ani

@@destroy:  DECR        R4
            MVO@        R0,         R4
            B           @@nxt_d

@@xpld_ani: MVII        #gfx.full,  R4          ; process animation

@@explod:   CALL        wait_nvbl
            DECLE       3

            CALL        load_gram
            DECLE       GRAM+5*8
            DECLE       4

            CMPI        #gfx.end_e, R4
            BNEQ        @@explod

@@scroll:   CALL        wait_nvbl               ; now, process scrolling
            DECLE       2

            MVII        #BT+227,    R3
            MVII        #$082F,     R0
            CLRR        R4

@@chk_s:    CMP@        R3,         R0          ; destroyed brick ? ...
            BEQ         @@scroll0

            SUBI        #20,        R3          ; ... no
            CMPI        #BT,        R3
            BGT         @@chk_s
            B           @@scroll3

@@scroll0:  MOVR        R3,         R2          ; ... yes
            INCR        R4

@@scroll1:  SUBI        #20,        R2          ; shift column
            MVI@        R2,         R1
            CMPI        #BT,        R2
            BGT         @@scroll2

            CLRR        R1

@@scroll2:  MVO@        R1,         R3
            SUBI        #20,        R3
            CMPI        #BT,        R3
            BGT         @@scroll1

@@scroll3:  ADDI        #241,       R3          ; next column
            CMPI        #BT+233,    R3
            BNEQ        @@chk_s

            TSTR        R4                      ; repeat ?
            BNEQ        @@scroll

            B           @@align                 ; look for new alignments

@@hc:       MVI         X,          R1          ; R1 = X

            MVI         $1FE,       R0          ; read controllers
            AND         $1FF,       R0
            XORI        #$FF,       R0
            BNEQ        @@chk_down

            MVO         R0,         HC_CNT
            INCR        R0
            MVO         R0,         FAST_OK
            B           @@sync

@@chk_down: MOVR        R0,         R3          ; down ? ...
            AND         FAST_OK,    R3
            BEQ         @@upd_cnt

            MVI         Y_CNT,      R3          ; ... yes : fast fall
            CMPI        #$D0,       R3
            BGE         @@upd_cnt

            MVII        #$D0,       R3
            MVO         R3,         Y_CNT

@@upd_cnt:  MVI         HC_CNT,     R3
            DECR        R3
            BMI         @@hc_ok

            MVO         R3,         HC_CNT
            B           @@sync

@@hc_ok:    MOVR        R0,         R3          ; button ? ...
            ANDI        #$E0,       R3
            BEQ         @@chk_r

            MVI         C0,         R3          ; ... yes : rotate
            MVI         C1,         R1
            MVI         C2,         R2
            MVO         R2,         C0
            MVO         R3,         C1
            MVO         R1,         C2
            B           @@hc_cnt

@@chk_r:    MOVR        R0,         R3          ; right ?
            ANDI        #2,         R3
            BNEQ        @@right

            ANDI        #8,         R0          ; left ?
            BEQ         @@sync

            SUBI        #8,         R1          ; x--
            SUBI        #61,        R2
            B           @@chk_dx

@@right     ADDI        #8,         R1          ; x++
            SUBI        #59,        R2

@@chk_dx:   CLRR        R0                      ; slide ok ?
            CMPI        #BT,        R2
            BLT         @@chk_dx1
            CMP@        R2,         R0
            BNEQ        @@hc_cnt
@@chk_dx1:  ADDI        #20,        R2
            CMPI        #BT,        R2
            BLT         @@chk_dx2
            CMP@        R2,         R0
            BNEQ        @@hc_cnt
@@chk_dx2:  ADDI        #20,        R2
            CMP@        R2,         R0
            BNEQ        @@hc_cnt

            MVO         R1,         X           ; save X

@@hc_cnt:   MVII        #12,        R0
            MVO         R0,         HC_CNT

@@sync:     CALL        wait_nvbl               ; VBL sync.
            DECLE       1

            MVII        #MOBX,      R4          ; update MOBX
            MVI         X,          R0
            ADDI        #$0240,     R0
            CALL        copy6

            MVII        #MOBY,      R4          ; update MOBY
            MVI         Y,          R0
            ADDI        #$00F8,     R0
            MOVR        R0,         R1
            CMPI        #$0100,     R0
            BGE         @@moby0

            CLRR        R1

@@moby0:    MVO@        R1,         R4
            MVO@        R1,         R4

            ADDI        #8,         R0
            MOVR        R0,         R1
            CMPI        #$0100,     R0
            BGE         @@moby2

            CLRR        R1

@@moby2:    MVO@        R1,         R4
            MVO@        R1,         R4

            ADDI        #8,         R0
            MVO@        R0,         R4
            MVO@        R0,         R4

            MVII        #MOBA,      R4          ; update MOBA
            MVII        #C0,        R5
            MVII        #3,         R2

@@moba:     MVI@        R5,         R1
            SLL         R1
            ADDI        #col.m,     R1
            MVI@        R1,         R0
            MVO@        R0,         R4
            INCR        R1
            MVI@        R1,         R0
            MVO@        R0,         R4

            DECR        R2
            BNEQ        @@moba

            B           loop

@@y2bt:     DECLE       BT+027, BT+047, BT+067, BT+087
            DECLE       BT+107, BT+127, BT+147, BT+167
            DECLE       BT+187, BT+207, BT+227, BT+247

            ENDP

; -------------------------------------------------------------------------- ;
; Game over                                                                  ;
; -------------------------------------------------------------------------- ;

game_over   PROC

            CALL        text
            DECLE       BT+108,7
            STRING16    "GAME"
            CALL        text
            DECLE       BT+128,7
            STRING16    "OVER"

            CALL        init_song
            DECLE       song01

            CALL        wait_nvbl
            DECLE       180

            CALL        EXEC_KEY
            B           intro

            ENDP

; ========================================================================== ;
; Routines                                                                   ;
; ========================================================================== ;

; -------------------------------------------------------------------------- ;
; [ upd_next ]                                                               ;
; -------------------------------------------------------------------------- ;
; Copy next brick to current brick and update next brick                     ;
; -------------------------------------------------------------------------- ;

upd_next    PROC
            PSHR        R5

            MVII        #C0,        R4
            MVII        #CN0,       R3
            MVII        #BT+116,    R2
            MVII        #RND1,      R5

@@update:   MVI@        R3,         R0

            MVI@        R5,         R1
            MVO@        R1,         R3

            ADDI        #col.c,     R1
            MVI@        R1,         R1
            MVO@        R1,         R2
            ADDI        #20,        R2

            MVO@        R0,         R4

            INCR        R3
            CMPI        #CN2,       R3
            BLE         @@update

            PULR        R7
            ENDP

; -------------------------------------------------------------------------- ;
; [ align ]                                                                  ;
; -------------------------------------------------------------------------- ;
; Look for bricks alignments                                                 ;
; -------------------------------------------------------------------------- ;

align       PROC

            MVI@        R5,         R0          ; R0 = step
            PSHR        R5

            MVI@        R1,         R4          ; R4 = initial brick

            TSTR        R4                      ; empty ? ...
            BEQ         @@done2

            ANDI        #$7FBF,     R4          ; ... no
            PSHR        R1

            CLRR        R5                      ; R5 = nb of identical bricks

@@check:    INCR        R5
            ADDR        R0,         R1

            CMPI        #BT+240,    R1          ; out of screen ? ...
            BGE         @@chk_ok

            MVI@        R1,         R2          ; ... no
            ANDI        #$7FBF,     R2
            CMPR        R2,         R4
            BEQ         @@check

@@chk_ok:   CMPI        #3,         R5          ; at least 3 bricks ? ...
            BLT         @@done1

@@mark:     SUBR        R0,         R1          ; ... yes : mark them
            MVI@        R1,         R2

            TSTR        R2                      ; already marked ? ...
            BMI         @@skip

            XORI        #$8040,     R2          ; ... no
            MVO@        R2,         R1
            INCR        R3

@@skip:     DECR        R5
            BNEQ        @@mark

@@done1:    PULR        R1
@@done2:    PULR        R7

            ENDP

; -------------------------------------------------------------------------- ;
; [ copy6 ]                                                                  ;
; -------------------------------------------------------------------------- ;
; Copy R0 at R4, R4+1, ..., R4+5                                             ;
; -------------------------------------------------------------------------- ;

copy6       PROC

            MVII        #6,         R3

@@copy:     MVO@        R0,         R4
            DECR        R3
            BNEQ        @@copy

            MOVR        R5,         R7

            ENDP

; -------------------------------------------------------------------------- ;
; [ draw ]                                                                   ;
; -------------------------------------------------------------------------- ;
; Draw consecutive GRAM cards                                                ;
; -------------------------------------------------------------------------- ;

draw        PROC

            MVI@        R5,         R4          ; R4 = backtab pointer
            MVI@        R5,         R1          ; R1 = nb of cards

@@draw:     MVO@        R0,         R4
            ADDI        #8,         R0
            DECR        R1
            BNEQ        @@draw

            MOVR        R5,         R7
            ENDP

; -------------------------------------------------------------------------- ;
; [ disp_score ]                                                             ;
; -------------------------------------------------------------------------- ;
; Display score                                                              ;
; -------------------------------------------------------------------------- ;

disp_score  PROC
            PSHR        R5

            MVI         SC_10K,     R0
            CALL        disp_dec
            DECLE       BT+40
            DECLE       disp_dec.2d
            MVI         SCORE,      R0
            CALL        disp_dec.0
            DECLE       disp_dec.4d

            PULR        R7
            ENDP

; -------------------------------------------------------------------------- ;
; [ disp_gems ]                                                              ;
; -------------------------------------------------------------------------- ;
; Display number of gems                                                     ;
; -------------------------------------------------------------------------- ;

disp_gems   PROC
            PSHR        R5

            MVI         GEMS,       R0
            CALL        disp_dec
            DECLE       BT+100
            DECLE       disp_dec.5d

            PULR        R7
            ENDP

; -------------------------------------------------------------------------- ;
; [ disp_level ]                                                             ;
; -------------------------------------------------------------------------- ;
; Display current level                                                      ;
; -------------------------------------------------------------------------- ;

disp_level  PROC
            PSHR        R5

            MVI         LEVEL,      R1
            MOVR        R1,         R0
            SLL         R1,         2
            ADDI        #$C,        R1
            MVO         R1,         Y_INC

            CALL        disp_dec
            DECLE       BT+160
            DECLE       disp_dec.2d

            PULR        R7
            ENDP

; -------------------------------------------------------------------------- ;
; [ disp_dec ]                                                               ;
; -------------------------------------------------------------------------- ;
; Display a decimal number                                                   ;
; -------------------------------------------------------------------------- ;

disp_dec    PROC

            MVI@        R5,         R4          ; R4 = backtab pointer
@@0:        MVI@        R5,         R3          ; R3 = pointer into conv. table

@@digit:    MVI@        R3,         R1

            MVII        #$3E4E,     R2          ; R2 = digit

@@conv:     ADDI        #8,         R2
            SUBR        R1,         R0
            BPL         @@conv

            MVO@        R2,         R4          ; write digit

            ADDR        R1,         R0          ; next one
            INCR        R3
            CMPI        #@@end,     R3
            BNEQ        @@digit

            MOVR        R5,         R7

@@5d:       DECLE       10000
@@4d:       DECLE       1000
@@3d:       DECLE       100
@@2d:       DECLE       10
@@1d:       DECLE       1
@@end:

            ENDP

; -------------------------------------------------------------------------- ;
; [ text ]                                                                   ;
; -------------------------------------------------------------------------- ;
; Draw a text                                                                ;
; -------------------------------------------------------------------------- ;

text        PROC

            MVI@        R5,         R4          ; R4 = backtab pointer
            MVI@        R5,         R2          ; R2 = attributes

@@read:     MVI@        R5,         R0
            MOVR        R0,         R1

            ANDI        #$FF,       R0          ; 1st char
            DECR        R0
            BMI         @@done

            SLL         R0,         2
            SLL         R0
            ADDR        R2,         R0
            MVO@        R0,         R4

            SWAP        R1                      ; 2nd char
            ANDI        #$FF,       R1
            DECR        R1
            BMI         @@done

            SLL         R1,         2
            SLL         R1
            ADDR        R2,         R1
            MVO@        R1,         R4
            B           @@read

@@done:     MOVR        R5,         R7

            ENDP

; -------------------------------------------------------------------------- ;
; [ cls ]                                                                    ;
; -------------------------------------------------------------------------- ;
; Clear screen                                                               ;
; -------------------------------------------------------------------------- ;

cls         PROC

            MVII        #BT,        R4
            MVII        #240,       R0
            B           EXEC_CLEAR

            ENDP

; -------------------------------------------------------------------------- ;
; [ clear_mobs ]                                                             ;
; -------------------------------------------------------------------------- ;
; Clear MOBs                                                                 ;
; -------------------------------------------------------------------------- ;

clear_mobs  PROC
            PSHR        R5

            CALL        wait_nvbl
            DECLE       1

            MVII        #MOBX,      R4
            MVII        #6,         R0
            PULR        R5
            B           EXEC_CLEAR

            ENDP

; -------------------------------------------------------------------------- ;
; [ clear ]                                                                  ;
; -------------------------------------------------------------------------- ;
; Clear memory area                                                          ;
; -------------------------------------------------------------------------- ;

clear       PROC

            MVI@        R5,         R4
            MVI@        R5,         R0
            B           EXEC_CLEAR

            ENDP

; -------------------------------------------------------------------------- ;
; [ init_gram ]                                                              ;
; -------------------------------------------------------------------------- ;
; Initialize GRAM                                                            ;
; -------------------------------------------------------------------------- ;

init_gram   PROC

            MVI@        R5,         R1          ; R1 = number of cards (x4)
            MVI@        R5,         R3          ; R3 = GRAM offset
            MVI@        R5,         R4          ; R4 = pointer to cards data
            PSHR        R5

@@loop:     MVII        #40,        R0          ; only 10 cards max. (i.e. 2x40 rows) per VBlank
            CMPR        R0,         R1
            BLE         @@load0

            SUBR        R0,         R1
            PSHR        R1
            MOVR        R0,         R1

            CALL        @@load
            MOVR        R5,         R3

            PULR        R1
            B           @@loop

; Load cards into GRAM
; --------------------------------------------------------------------------

@@load:     PSHR        R5

@@load0:    PSHR        R3
            CALL        wait_nvbl
            DECLE       1
            PULR        R5

            B           load_gram.load

            ENDP

; -------------------------------------------------------------------------- ;
; [ load_gram ]                                                              ;
; -------------------------------------------------------------------------- ;
; Load cards into GRAM                                                       ;
; -------------------------------------------------------------------------- ;

load_gram   PROC

            MVI@        R5,         R0
            MVI@        R5,         R1
            PSHR        R5
            MOVR        R0,         R5

@@load:     MVI@        R4,         R0
            MVO@        R0,         R5
            SWAP        R0
            MVO@        R0,         R5
            DECR        R1
            BNEQ        @@load

            PULR        R7
            ENDP

; -------------------------------------------------------------------------- ;
; [ wait_nvbl ]                                                              ;
; -------------------------------------------------------------------------- ;
; Wait for N VBlank interrupts                                               ;
; -------------------------------------------------------------------------- ;

wait_nvbl   PROC
            PSHR        R5

            MVII        #5,         R0
            CALL        EXEC_RND
            MVO         R0,         RND1
            MVII        #5,         R0
            CALL        EXEC_RND
            MVO         R0,         RND2
            MVII        #5,         R0
            CALL        EXEC_RND
            MVO         R0,         RND3

            PULR        R5
            MVI@        R5,         R0
            PSHR        R5

            PSHR        R1
            PSHR        R3
            PSHR        R4

@@loop:     PSHR        R0
            CALL        play                    ; play music

            MVI         FRAME,      R2

@@wait:     CMP         FRAME,      R2
            BEQ         @@wait

            PULR        R0
            DECR        R0
            BNEQ        @@loop

            PULR        R4
            PULR        R3
            PULR        R1

            PULR        R7
            ENDP

; -------------------------------------------------------------------------- ;
; [ vblank ]                                                                 ;
; -------------------------------------------------------------------------- ;
; VBlank routine                                                             ;
; -------------------------------------------------------------------------- ;

vblank      PROC

            MVO         R0,         $0020       ; enable display

@@0:        MVI         FRAME,      R0          ; frame++
            INCR        R0
            MVO         R0,         FRAME

            B           RET_INT
            ENDP

; ========================================================================== ;
; Data                                                                       ;
; ========================================================================== ;

; -------------------------------------------------------------------------- ;
; Keypad index                                                               ;
; -------------------------------------------------------------------------- ;

keypad      PROC

            DECLE       $81, $41, $21, $82, $42, $22, $84, $44, $24

            ENDP

; -------------------------------------------------------------------------- ;
; Bricks colors                                                              ;
; -------------------------------------------------------------------------- ;

col         PROC

            ; FG/BG cards
@@c:        DECLE       $1C06                   ; yellow / orange
            DECLE       $3C0D                   ; yellow-green / green
            DECLE       $3E02                   ; red / purple
            DECLE       $1A09                   ; cyan / blue
            DECLE       $1807                   ; white / grey

            ; MOBs
@@m:        DECLE       $0806, $1822            ; yellow / orange
            DECLE       $1806, $0825            ; yellow-green / green
            DECLE       $0802, $1827            ; red / purple
            DECLE       $1801, $0821            ; cyan / blue
            DECLE       $0807, $1820            ; white / grey

            ENDP

; -------------------------------------------------------------------------- ;
; Graphics                                                                   ;
; -------------------------------------------------------------------------- ;

gfx         PROC

            ; 00-01 / bricks
@@b0:       DECLE       $BE7F, $E8DC, $E4F8, $81C2
            DECLE       $4180, $1723, $1B07, $7E3D

            ; 02-03 / borders
            DECLE       $0505, $0505, $0505, $0505
            DECLE       $A0A0, $A0A0, $A0A0, $A0A0

            ; 04 / full
@@full:     DECLE       $FFFF, $FFFF, $FFFF, $FFFF

            ; 05 / explosion
            DECLE       $FF7E, $FFFF, $FFFF, $7EFF
            DECLE       $7E3C, $FFFF, $FFFF, $3C7E
            DECLE       $3C18, $FF7E, $7EFF, $183C
            DECLE       $1800, $7E3C, $3C7E, $0018
            DECLE       $0000, $3C18, $183C, $0000
            DECLE       $0000, $1800, $0018, $0000
            DECLE       $0000, $2418, $1824, $0000
            DECLE       $1800, $4200, $0042, $0018
            DECLE       $0018, $8100, $0081, $1800
            DECLE       $0000, $0000, $0000, $0000
@@end_e:
            ; 08-09 / bricks (mirror)
@@b1:       DECLE       $4301, $1F27, $3B17, $FE7D
            DECLE       $BC7E, $E0D8, $C4E8, $0182

            ; 10-19 / digits
@@digits:   DECLE       $3E00, $7777, $7777, $3E77
            DECLE       $1C00, $1C3C, $1C1C, $7F1C
            DECLE       $7E00, $0707, $703E, $3F70
            DECLE       $7E00, $0707, $071F, $7E07
            DECLE       $7000, $7676, $067F, $0606
            DECLE       $7F00, $7070, $077E, $7E07
            DECLE       $3F00, $7070, $777E, $3E77
            DECLE       $7F00, $0707, $1C0E, $1C1C
            DECLE       $3E00, $7777, $773E, $3E77
            DECLE       $3E00, $7777, $073F, $7E07

            ; 20-22 / 'Score'
@@texts:    DECLE       $603C, $7860, $0D3D, $780D
            DECLE       $0000, $C900, $1515, $C915
            DECLE       $0000, $8C00, $9850, $4C50

            ; 23-26 / 'Jewels'
            DECLE       $0C3F, $0C0C, $0D0D, $384D
            DECLE       $0000, $D100, $9511, $D11B
            DECLE       $0000, $3400, $6444, $3744
            DECLE       $0000, $3000, $2040, $6010

            ; 27-29 / 'Level'
            DECLE       $6060, $6060, $6161, $7C61
            DECLE       $0000, $D400, $9515, $C815
            DECLE       $0000, $D000, $9010, $DC10

            ; 30-36 / title
            DECLE       $C07C, $78C1, $0C0C, $00F8
            DECLE       $C040, $C1E7, $CDC7, $0067
            DECLE       $0000, $B01E, $B0B0, $009E
            DECLE       $C0C0, $D8CC, $D8F0, $00CC
            DECLE       $D8CF, $1E98, $1818, $000F
            DECLE       $0000, $667F, $6666, $0066
            DECLE       $0707, $67C7, $6066, $0066

;            ; small digits (for MOBs)
;@@s_digits: DECLE       $AAEE, $AAAA, $AAAA, $00EE
;            DECLE       $CC44, $4444, $4444, $00EE
;            DECLE       $22EE, $EE22, $8888, $00EE
;            DECLE       $22EE, $6622, $2222, $00EE
;            DECLE       $8888, $EEAA, $2222, $0022
;            DECLE       $88EE, $EE88, $2222, $00EE
;            DECLE       $88EE, $EE88, $AAAA, $00EE
;            DECLE       $22EE, $2222, $2222, $0022
;            DECLE       $AAEE, $EEAA, $AAAA, $00EE
;            DECLE       $AAEE, $EEAA, $2222, $00EE

            ENDP

; ========================================================================== ;
; External routines & data                                                   ;
; ========================================================================== ;

            INCLUDE     "tracker.asm"
            INCLUDE     "music.asm"

_EOF_:
